#!/usr/bin/python3

import apt_pkg
import logging
import unittest

apt_pkg.config.set("Dir", "./aptroot")

import unattended_upgrade
from unattended_upgrade import (
    check_changes_for_sanity,
    is_in_allowed_origin,
    get_distro_codename,
    match_whitelist_string,
    UnknownMatcherError,
)

from typing import List


class MockOrigin():
    trusted = True


class MockCandidate():
    pass


class MockPackage():
    pass


class MockCache(dict):
    def __iter__(self):
        for pkgname in self.keys():
            yield self[pkgname]

    def get_changes(self):
        for pkgname in self.keys():
            yield self[pkgname]
    allowed_origins = []  # type: List[str]
    blacklist = []        # type: List[str]
    whitelist = []        # type: List[str]
    strict_whitelist = False  # type: bool


class MockDepCache():
    pass


class TestOriginPatern(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_match_whitelist_string(self):
        origin = self._get_mock_origin(
            "OriginUbuntu", "LabelUbuntu", "ArchiveUbuntu",
            "archive.ubuntu.com", "main")
        # good
        s = "o=OriginUbuntu"
        self.assertTrue(match_whitelist_string(s, origin))
        s = "o=OriginUbuntu,l=LabelUbuntu,a=ArchiveUbuntu," \
            "site=archive.ubuntu.com"
        self.assertTrue(match_whitelist_string(s, origin))
        # bad
        s = ""
        self.assertFalse(match_whitelist_string(s, origin))
        s = "o=something"
        self.assertFalse(match_whitelist_string(s, origin))
        s = "o=LabelUbuntu,a=no-match"
        self.assertFalse(match_whitelist_string(s, origin))
        # with escaping
        origin = self._get_mock_origin("Google, Inc.", archive="stable")
        # good
        s = "o=Google\\, Inc.,a=stable"
        self.assertTrue(match_whitelist_string(s, origin))

    def test_match_whitelist_from_conffile(self):
        # read some
        apt_pkg.config.clear("Unattended-Upgrade")
        apt_pkg.read_config_file(
            apt_pkg.config, "./data/50unattended-upgrades.Test")
        allowed_origins = unattended_upgrade.get_allowed_origins()
        #print allowed_origins
        self.assertTrue("o=aOrigin,a=aArchive" in allowed_origins)
        self.assertTrue("s=aSite,l=aLabel" in allowed_origins)
        self.assertTrue("o=Google\\, Inc.,suite=stable" in allowed_origins)

    def test_macro(self):
        codename = get_distro_codename()
        s = "a=${distro_codename}"
        origin = self._get_mock_origin("Foo", archive=codename)
        self.assertTrue(match_whitelist_string(s, origin))

    def test_compatiblity(self):
        apt_pkg.config.clear("Unattended-Upgrade")
        apt_pkg.read_config_file(
            apt_pkg.config, "./data/50unattended-upgrades.compat")
        allowed_origins = unattended_upgrade.get_allowed_origins()
        #print allowed_origins
        self.assertTrue("o=Google\\, Inc.,a=stable" in allowed_origins)
        self.assertTrue("o=MoreCorp\\, eink,a=stable" in allowed_origins)
        # test whitelist
        pkg = self._get_mock_package()
        self.assertTrue(is_in_allowed_origin(pkg.candidate, allowed_origins))

    def test_escaped_colon(self):
        apt_pkg.config.clear("Unattended-Upgrade")
        apt_pkg.read_config_file(
            apt_pkg.config, "./data/50unattended-upgrades.colon")
        allowed_origins = unattended_upgrade.get_allowed_origins()

        self.assertIn('o=http://foo.bar,a=stable', allowed_origins)

    def test_unkown_matcher(self):
        apt_pkg.config.clear("Unattended-Upgrade")
        s = "xxx=OriginUbuntu"
        with self.assertRaises(UnknownMatcherError):
            self.assertTrue(match_whitelist_string(s, None))

    def test_blacklist(self):
        # get the mocks
        pkg = self._get_mock_package("linux-image")
        cache = self._get_mock_cache()
        cache[pkg.name] = pkg
        # origins and blacklist
        cache.allowed_origins = ["o=Ubuntu"]
        cache.blacklist = ["linux-.*"]
        cache.Whitelist = [".*"]
        # with blacklist pkg
        self.assertFalse(
            check_changes_for_sanity(cache))
        # with "normal" pkg
        pkg.name = "apt"
        self.assertTrue(
            check_changes_for_sanity(cache))

    def test_whitelist_with_strict_whitelisting(self):
        cache = self._get_mock_cache()
        for pkgname in ["not-whitelisted", "whitelisted"]:
            pkg = self._get_mock_package(name=pkgname)
            cache[pkg.name] = pkg
        # origins and blacklist
        cache.allowed_origins = ["o=Ubuntu"]
        cache.whitelist = ["whitelisted"]
        # test with strict whitelist
        cache.strict_whitelist = True
        # ensure that a not-whitelisted pkg will fail
        self.assertTrue(cache["not-whitelisted"].marked_upgrade)
        self.assertFalse(
            check_changes_for_sanity(cache))

    def _get_mock_cache(self):
        cache = MockCache()
        cache._depcache = MockDepCache()
        cache._depcache.broken_count = 0
        cache.install_count = 1
        return cache

    def _get_mock_origin(self, aorigin="", label="", archive="",
                         site="", component=""):
        origin = MockOrigin()
        origin.origin = aorigin
        origin.label = label
        origin.archive = archive
        origin.site = site
        origin.component = component
        return origin

    def _get_mock_package(self, name="foo"):
        pkg = MockPackage()
        pkg._pkg = MockPackage()
        pkg._pkg.selected_state = 0
        pkg.name = name
        pkg.marked_install = True
        pkg.marked_upgrade = True
        pkg.marked_delete = False
        pkg.candidate = MockCandidate()
        pkg.candidate.origins = [self._get_mock_origin("Ubuntu"),
                                 self._get_mock_origin(aorigin="Google, Inc.",
                                                       archive="stable")]
        pkg.candidate.record = {}
        return pkg

    def test_match_whitelist_wildcard(self):
        origin = self._get_mock_origin(
            "OriginUbuntu", "LabelUbuntu", "ArchiveUbuntu",
            "archive.ubuntu.com", "main")
        # good
        s = "o=OriginU*"
        self.assertTrue(match_whitelist_string(s, origin))
        # bad
        s = "o=X*"
        self.assertFalse(match_whitelist_string(s, origin))
        # good
        s = "o=?riginUbunt?"
        self.assertTrue(match_whitelist_string(s, origin))
        # good
        s = "o=*Ubunt?"
        self.assertTrue(match_whitelist_string(s, origin))

    def test_get_allowed_origins_legacy(self):
        for cfg, (distro_id, distro_codename) in (
                # ":" as separator
                ("Ubuntu:lucid-security", ("Ubuntu", "lucid-security")),
                ("http\\://foo.bar:stable", ("http://foo.bar", "stable")),
                # space as separator
                ("Ubuntu lucid-security", ("Ubuntu", "lucid-security")),
                ("http\\://baz.mee stable", ("http://baz.mee", "stable"))):
            apt_pkg.config.clear("Unattended-Upgrade::Allowed-Origins")
            apt_pkg.config.set("Unattended-Upgrade::Allowed-Origins::", cfg)
            li = unattended_upgrade.get_allowed_origins_legacy()
            self.assertEqual(len(li), 1)
            self.assertEqual(li[0], "o=%s,a=%s" % (distro_id, distro_codename))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import logging
import os
import sys
import tempfile
import unittest

sys.path.insert(0, "..")

from unattended_upgrade import _setup_logging


class MockOptions:
    dry_run = False
    debug = False
    verbose = False
    apt_debug = False


class TestLogInstallProgress(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        apt_pkg.init()
        self.mock_options = MockOptions()

    def test_log_installprogress(self):
        logdir = os.path.join(self.tempdir, "mylog")
        apt_pkg.config.set("Unattended-Upgrade::LogDir", logdir)
        logging.root.handlers = []
        _setup_logging(self.mock_options)
        self.assertTrue(os.path.exists(logdir))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
"""Test unattended_upgrades against the real archive in a chroot.

Note that this test is not run by the makefile in this folder, as it requires
network access, and it fails in some situations (unclear which).
"""

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt
import glob
import logging
import os
import re
import unittest

import unattended_upgrade


apt_pkg.config.set("APT::Architecture", "amd64")


class MockOptions():
    def __init__(self, debug=True, dry_run=False):
        self.debug = debug
        self.dry_run = dry_run
        self.apt_debug = False
        self.minimal_upgrade_steps = True
        self.verbose = False


class TestAgainstRealArchive(unittest.TestCase):

    def setUp(self):
        for g in ["./aptroot/var/log/apt/*",
                  "./aptroot/var/log/*"]:
            for f in glob.glob(g):
                if os.path.isfile(f):
                    os.remove(f)
        # get a lucid based cache (test good until 04/2015)
        cache = apt.Cache(rootdir="./aptroot")
        cache.update()
        del cache
        # ensure apt does not do any post-invoke stuff that fails
        # (because we are not root)
        apt_pkg.config.clear("DPkg::Post-Invoke")
        apt_pkg.config.clear("DPkg::Pre-Invoke")
        unattended_upgrade.DISTRO_CODENAME = "lucid"

    def test_against_real_archive(self):
        # create mock options
        options = MockOptions(dry_run=False, debug=True)
        # run unattended-upgrades against fake system
        logdir = os.path.abspath("./aptroot/var/log/")
        logfile = os.path.join(logdir, "unattended-upgrades.log")
        apt_pkg.config.set("APT::UnattendedUpgrades::LogDir", logdir)

        # main
        res = unattended_upgrade.main(options, os.path.abspath("./aptroot"))
        logging.debug(res)
        # check if the log file exists
        self.assertTrue(os.path.exists(logfile))
        with open(logfile) as fp:
            log = fp.read()
        # check that stuff worked
        self.assertFalse(" ERROR " in log, log)
        # check if we actually have the expected ugprade in it
        self.assertTrue(
            re.search("INFO Packages that will be upgraded:.*awstats", log))
        # apt-doc has a higher version in -updates than in -security
        # and no other dependencies so its a perfect test
        self.assertTrue(
            re.search("INFO Packages that will be upgraded:.*apt-doc", log))
        self.assertFalse(
            re.search("INFO Packages that will be upgraded:.*ant-doc", log))
        self.assertTrue(
            re.search("DEBUG skipping blacklisted package 'ant-doc'", log))
        # test dpkg install log
        #term_log = open("aptroot/var/log/apt/term.log").read()
        # FIXME: when we redirect STDIN the below test will break - however
        #        we need to redirect it as otherwise we may hang forever
        #        - this is actually a bug in apt that uses "tcgetattr(0, &tt)"
        #          on FD=0 instead of FD=1
        #print term_log
        #self.assertTrue(
        #    re.search(
        #        "fake-dpkg: --status-fd .* --configure.*awstats", term_log))


if __name__ == "__main__":
    import locale
    locale.setlocale(locale.LC_ALL, "C")
    unittest.main()



import os
import subprocess
import unittest


class TestPyflakesClean(unittest.TestCase):
    """ ensure that the tree is pyflakes clean """

    def test_pyflakes_clean(self):
        top_src_dir = os.path.join(os.path.dirname(__file__), "..")
        targets = [
            top_src_dir,
            os.path.join(top_src_dir, "unattended-upgrade"),
            os.path.join(top_src_dir, "unattended-upgrade-shutdown"),
        ]
        self.assertEqual(subprocess.call(["pyflakes3", ] + targets), 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

import apt_pkg
import logging
import re
import subprocess
import unittest

from unattended_upgrade import (running_kernel_pkgs_regexp,
                                versioned_kernel_pkgs_regexp)


class TestKernelPatterns(unittest.TestCase):

    def test_versioned(self):
        """kernel package patterns should cover versioned packages"""
        versioned_regexp = versioned_kernel_pkgs_regexp()
        running_regexp = running_kernel_pkgs_regexp()
        running_kernel_version = subprocess.check_output(
            ["uname", "-r"], universal_newlines=True).rstrip()
        running_escaped_regexp = ".*" + re.escape(running_kernel_version)
        try:
            running_noflavor_regexp = "linux.*-" + re.escape(
                re.match("[1-9][0-9]*\\.[0-9]+\\.[0-9]+-[0-9]+",
                         running_kernel_version)[0])
        except TypeError:
            self.skipTest("Could not find flavor of running kernel. It may "
                          "not be a packaged one.")

        cache = apt_pkg.Cache(None)
        not_matched = set()
        for pkg in cache.packages:
            pkg_name = pkg.name
            if re.match(running_noflavor_regexp, pkg_name):
                if re.match(running_escaped_regexp, pkg_name) \
                   and not re.match(running_regexp, pkg_name):
                    logging.debug("Package %s matched %s and %s, "
                                  "but did not match %s",
                                  pkg_name, running_noflavor_regexp,
                                  running_escaped_regexp,
                                  running_regexp)
                    not_matched.add(pkg_name)
                    continue
                if not re.match(versioned_regexp, pkg_name):
                    logging.debug("Package %s matched %s, "
                                  "but did not match %s",
                                  pkg_name, running_noflavor_regexp,
                                  versioned_regexp)
                    not_matched.add(pkg.name)

        self.assertTrue(not not_matched,
                        "kernel packages not matched: %s"
                        % " ".join(not_matched))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import os
import unittest

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt

import unattended_upgrade

apt.apt_pkg.config.set("APT::Architecture", "amd64")


class MockOptions(object):
    debug = False
    verbose = False
    download_only = False
    dry_run = True
    apt_debug = False
    minimal_upgrade_steps = True


class TestRewindCache(unittest.TestCase):

    def setUp(self):
        rootdir = os.path.abspath("./root.rewind")
        dpkg_status = os.path.abspath(
            os.path.join(rootdir, "var", "lib", "dpkg", "status"))
        apt.apt_pkg.config.set("Dir::State::status", dpkg_status)
        apt.apt_pkg.config.set(
            "Unattended-Upgrade::Allow-APT-Mark-Fallback", "true")
        self.cache = unattended_upgrade.UnattendedUpgradesCache(
            rootdir=rootdir)

    def test_rewind_cache(self):
        """ Test that rewinding the cache works correctly, debian #743594 """
        options = MockOptions()
        to_upgrade = unattended_upgrade.calculate_upgradable_pkgs(
            self.cache, options)
        self.assertEqual(to_upgrade, [self.cache[p] for p
                                      in ["test-package", "test2-package",
                                          "test3-package"]])
        unattended_upgrade.rewind_cache(self.cache, to_upgrade)
        self.assertEqual(self.cache['test-package'].candidate.version, "2.0")


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3


import os
import shutil
import tempfile
import unittest

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt
import unattended_upgrade

try:
    from typing import List
    List   # pyflaks
except ImportError:
    pass


class MockFetcher:
    items = []  # type: List[MockAcquireItem]


class MockAcquireItem:
    def __init__(self, destfile):
        self.destfile = destfile


class TestClean(unittest.TestCase):

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tempdir)
        os.chdir(self.tempdir)

    def test_clean(self):
        apt.apt_pkg.config.set("dir::cache::archives", self.tempdir)
        os.makedirs("dir")
        with open("file1", "w"):
            pass
        with open("file2", "w"):
            pass
        with open("lock", "w"):
            pass
        fetcher = MockFetcher()
        fetcher.items = [MockAcquireItem("file1"), MockAcquireItem("file2")]
        unattended_upgrade.clean_downloaded_packages(fetcher)
        # files get removed
        self.assertEqual(os.path.exists("file1"), False)
        self.assertEqual(os.path.exists("file2"), False)
        # exluces are honored
        self.assertEqual(os.path.exists("lock"), True)
        # dirs are untouched
        self.assertEqual(os.path.exists("dir"), True)


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt
import os
import shutil
import tempfile
import unittest

try:
    from typing import List, Tuple
    List   # pyflaks
    Tuple  # pyflakes
except ImportError:
    pass

import unattended_upgrade


class LogInstallProgressMock(unattended_upgrade.LogInstallProgress):

    # klass data so that we can verify in the test as the actual
    # object is destroyed
    DATA = []  # type: List[Tuple[str, float]]

    # overwrite to log the data
    def status_change(self, pkg, percent, status):
        print(pkg, percent)
        self.DATA.append([pkg, percent])


class TestMinimalPartitions(unittest.TestCase):

    def setUp(self):
        # setup dry-run mode for apt
        apt_pkg.config.set("Dir", "./aptroot")
        apt_pkg.config.set("Dir::Cache", "/tmp")
        apt_pkg.config.set("Debug::NoLocking", "1")
        apt_pkg.config.set("Debug::pkgDPkgPM", "1")
        apt_pkg.config.set("Dir::State::extended_states", "./extended_states")
        apt_pkg.config.clear("Dpkg::Post-Invoke")
        apt_pkg.config.clear("Dpkg::Pre-Install-Pkgs")
        self.cache = apt.Cache()
        # for the log
        self.tempdir = tempfile.mkdtemp()
        self.addCleanup(lambda: shutil.rmtree(self.tempdir))
        apt_pkg.config.set("Unattended-Upgrade::LogDir", self.tempdir)

    def tearDown(self):
        if os.path.exists("./extended_states"):
            os.remove("./extended_states")

    def test_upgrade_in_minimal_steps(self):
        self.cache.upgrade(True)
        # upgrade only a tiny subset in the test
        pkgs_to_upgrade = [
            pkg.name for pkg in self.cache.get_changes()][:5]
        unattended_upgrade.PROGRESS_LOG = \
            "./aptroot/var/run/unatteded-upgrades.progress"
        unattended_upgrade.LogInstallProgress = LogInstallProgressMock
        unattended_upgrade.upgrade_in_minimal_steps(
            self.cache, pkgs_to_upgrade, "",
            os.path.join(self.tempdir, "mylog"))
        # ensure we count upwarts
        last_percent = -1
        for (pkg, percent) in LogInstallProgressMock.DATA:
            self.assertTrue(last_percent < percent)
            last_percent = percent
        # cleanup class data
        LogInstallProgressMock.DATA = []


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from email.parser import Parser
from io import StringIO
import logging
from mock import patch
import os
import shutil
import sys
import tempfile
from textwrap import dedent
import unittest


import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")

import unattended_upgrade
from unattended_upgrade import (
    get_dpkg_log_content,
    LoggingDateTime,
    send_summary_mail,
    setup_apt_listchanges,
)


class MockOptions(object):
    debug = False
    verbose = False
    download_only = False
    dry_run = False
    apt_debug = False
    minimal_upgrade_steps = True


class ExtractDpkgLogTestCase(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        self.addCleanup(logging.shutdown)

    def test_get_dpkg_log_content(self):
        logfile_dpkg = os.path.join(self.tmpdir, "apt-term.log")
        # note that we intentionally not have a "Log ended:" here
        # because this may happen if something crashes power goes
        # down etc
        OLD_LOG = dedent("""\
            Log started: 2013-01-01  12:00:00
            old logfile text
        """)
        NEW_LOG = dedent("""\
            Log started: 2014-10-28  10:00:00
            random logfile_dpkg text
            Log ended: 2013-01-01  12:20:00

            Log started: 2014-10-28  12:21:00
            more random logfile_dpkg text
            Log ended: 2013-01-01  12:30:00
            """)
        with open(logfile_dpkg, "w") as fp:
            fp.write(OLD_LOG)
            fp.write("\n")
            fp.write(NEW_LOG)
        start_time = LoggingDateTime.from_string("2014-10-28  10:00:00")
        dpkg_log_content = get_dpkg_log_content(logfile_dpkg, start_time)
        self.assertEqual(dpkg_log_content, NEW_LOG)


# note this is not a unittest.TestCase as it needs to be parameterized
class CommonTestsForMailxAndSendmail(object):

    EXPECTED_MAIL_CONTENT_STRINGS = [
        "random logfile_dpkg text",
        "mem_log text",
    ]
    NOT_EXPECTED_MAIL_CONTENT_STRINGS = [
        "old logfile text",
    ]

    def common_setup(self):
        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        self.addCleanup(logging.shutdown)
        logging.root.handlers = []
        # monkey patch to make it testable
        unattended_upgrade.REBOOT_REQUIRED_FILE = os.path.join(
            self.tmpdir, "reboot-required")
        unattended_upgrade.MAIL_BINARY = "./no-mailx-binary-here"
        unattended_upgrade.SENDMAIL_BINARY = "./no-sendmail-binary-here"
        # setup mail
        apt_pkg.config.set("Unattended-Upgrade::Mail", "root")
        apt_pkg.config.set("Unattended-Upgrade::MailReport", "on-change")
        apt_pkg.config.set("Unattended-Upgrade::LogDir", self.tmpdir)

    def _return_mock_data(self, successful=True):
        """ return input tuple for send_summary_mail """
        pkgs = ["2vcard"]
        res = successful
        result_str = "Result String"
        pkgs_kept_back = {"Debian wheezy-security": ["linux-image"]}
        pkgs_removed = ["telnet"]
        pkgs_kept_installed = ["hello"]
        # include some unicode chars here for good measure
        mem_log = StringIO("""mem_log text üöä
Allowed origins are: ['o=Debian,n=wheezy', 'o=Debian,n=wheezy-updates',\
 'o=Debian,n=wheezy,l=Debian-Security', 'origin=Debian,archive=stable,label=\
Debian-Security']
""")
        dpkg_log_content = dedent("""\
        Log started: 2014-10-28  12:21:00
        random logfile_dpkg text
        Log ended: 2013-01-01  12:30:00
        """)
        return (pkgs, res, result_str, pkgs_kept_back, pkgs_removed,
                pkgs_kept_installed, mem_log, dpkg_log_content)

    def _verify_common_mail_content(self, mail_txt):
        for expected_string in self.EXPECTED_MAIL_CONTENT_STRINGS:
            self.assertTrue(expected_string in mail_txt)
        for not_expected_string in self.NOT_EXPECTED_MAIL_CONTENT_STRINGS:
            self.assertFalse(not_expected_string in mail_txt)
        self.assertEqual(mail_txt.count("Log started: "), 1)

    def test_summary_mail_reboot(self):
        with open(unattended_upgrade.REBOOT_REQUIRED_FILE, "w") as fp:
            fp.write("")
        send_summary_mail(*self._return_mock_data())
        # this is used for py2 compat for py3 only we can do
        # remove the "rb" and the subsequent '.decode("utf-8")'
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue("[reboot required]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue("Packages that were upgraded:\n 2vcard" in mail_txt)

    def test_summary_mail_no_reboot(self):
        send_summary_mail(*self._return_mock_data())
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertFalse("[reboot required]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue("Packages that were upgraded:\n 2vcard" in mail_txt)

    def test_summary_mail_only_on_error(self):
        # default is to always send mail, ensure this is correct
        # for both success and failure
        apt_pkg.config.set("Unattended-Upgrade::MailReport", "on-change")
        send_summary_mail(*self._return_mock_data(successful=True))
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            self._verify_common_mail_content(fp.read().decode("utf-8"))
        os.remove(os.path.join(self.tmpdir, "mail.txt"))
        # now with a simulated failure
        send_summary_mail(*self._return_mock_data(successful=False))
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            self._verify_common_mail_content(fp.read().decode("utf-8"))
        os.remove(os.path.join(self.tmpdir, "mail.txt"))
        # now test with "only-on-error"
        apt_pkg.config.set("Unattended-Upgrade::MailReport", "only-on-error")
        send_summary_mail(*self._return_mock_data(successful=True))
        self.assertFalse(
            os.path.exists(os.path.join(self.tmpdir, "mail.txt")))
        send_summary_mail(*self._return_mock_data(successful=False))
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self._verify_common_mail_content(mail_txt)
        self.assertTrue("Unattended upgrade result: Result String" in mail_txt)
        self.assertTrue(
            os.path.exists(os.path.join(self.tmpdir, "mail.txt")))
        self.assertTrue(
            "Packages that attempted to upgrade:\n 2vcard" in mail_txt)

    def test_summary_mail_blacklisted(self):
        # Test that blacklisted packages are mentioned in the mail message.
        send_summary_mail(*self._return_mock_data())
        self.assertTrue(
            os.path.exists(os.path.join(self.tmpdir, "mail.txt")))
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue("[package on hold]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue(
            "Packages with upgradable origin but kept back:\n"
            " Debian wheezy-security:\n  linux-image"
            in mail_txt)

    def test_summary_mail_blacklisted_only(self):
        # Test that when only blacklisted packages are available, they
        # are still mentioned in the mail message.
        pkgs, res, result_str, pkgs_kept_back, pkgs_removed, \
            pkgs_kept_installed, mem_log, logf_dpkg = self._return_mock_data(
                successful=True)
        pkgs = []
        send_summary_mail(pkgs, res, result_str, pkgs_kept_back, pkgs_removed,
                          pkgs_kept_installed, mem_log, logf_dpkg)
        self.assertTrue(
            os.path.exists(os.path.join(self.tmpdir, "mail.txt")))
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue("[package on hold]" in mail_txt)
        self._verify_common_mail_content(mail_txt)
        self.assertTrue(
            "Packages with upgradable origin but kept back:\n"
            " Debian wheezy-security:\n  linux-image"
            in mail_txt)
        self.assertFalse(
            "Packages that attempted to upgrade:\n 2vcard" in mail_txt)

    def test_apt_listchanges(self):
        # test with sendmail available
        unattended_upgrade.SENDMAIL_BINARY = "/bin/true"
        setup_apt_listchanges("./data/listchanges.conf.mail")
        self.assertEqual(os.environ["APT_LISTCHANGES_FRONTEND"], "mail")
        # test without sendmail
        unattended_upgrade.SENDMAIL_BINARY = "/bin/not-here-xxxxxxxxx"
        setup_apt_listchanges("./data/listchanges.conf.pager")
        self.assertEqual(os.environ["APT_LISTCHANGES_FRONTEND"], "none")

    def test_summary_mail_from_address(self):
        apt_pkg.config.set("Unattended-Upgrade::Sender", "rootolv")
        self.addCleanup(apt_pkg.config.set, "Unattended-Upgrade::Sender", "")
        send_summary_mail(*self._return_mock_data())
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue(
            "From: rootolv" in mail_txt, "missing From: in %s" % mail_txt)

    @patch("unattended_upgrade.run")
    def test_exception(self, mock_run):
        exception_string = "Test exception for email"
        mock_run.side_effect = Exception(exception_string)
        # run it
        options = MockOptions()
        unattended_upgrade.LOCK_FILE = "./u-u.lock"
        exception_raised = False
        try:
            unattended_upgrade.main(options)
        except Exception as e:
            self.assertEqual(str(e), exception_string)
            exception_raised = True
        self.assertTrue(exception_raised)
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            mail_txt = fp.read().decode("utf-8")
        self.assertTrue(exception_string in mail_txt)


class MailxTestCase(CommonTestsForMailxAndSendmail, unittest.TestCase):

    def setUp(self):
        self.common_setup()
        unattended_upgrade.MAIL_BINARY = make_mock_mailx(self.tmpdir)

    def _verify_common_mail_content(self, mail_txt):
        CommonTestsForMailxAndSendmail._verify_common_mail_content(
            self, mail_txt)
        # setting this header with mailx is not possible so ensure
        # we don't accidently try
        self.assertFalse('text/plain; charset="utf-8"' in mail_txt)


def make_mock_sendmail(tmpdir):
    p = os.path.join(tmpdir, "mock-sendmail")
    with open(p, "w") as fp:
        fp.write("""#!/bin/sh
cat - -- > %s/mail.txt
    """ % tmpdir)
    os.chmod(p, 0o755)
    return p


def make_mock_mailx(tmpdir):
    p = os.path.join(tmpdir, "mock-mail")
    with open(p, "w") as fp:
        fp.write("""#!/bin/sh

echo "From: $2" > %(tmp)s/mail.txt
echo "Subject: $4" >> %(tmp)s/mail.txt
cat - -- >> %(tmp)s/mail.txt
        """ % {'tmp': tmpdir})
    os.chmod(p, 0o755)
    return p


class SendmailTestCase(CommonTestsForMailxAndSendmail, unittest.TestCase):

    def setUp(self):
        self.common_setup()
        unattended_upgrade.SENDMAIL_BINARY = make_mock_sendmail(self.tmpdir)

    def _verify_common_mail_content(self, mail_txt):
        CommonTestsForMailxAndSendmail._verify_common_mail_content(
            self, mail_txt)

        # python2 needs this as utf8 encoded string (not unicode)
        if sys.version < '3':
            mail_txt = mail_txt.encode("utf-8")

        msg = Parser().parsestr(mail_txt)
        content_type = msg["Content-Type"]
        self.assertEqual(content_type, 'text/plain; charset="utf-8"')

    def test_mail_quoted_printable(self):
        """Regression test for debian bug #700178"""
        send_summary_mail(*self._return_mock_data())
        with open(os.path.join(self.tmpdir, "mail.txt"), "rb") as fp:
            log_data = fp.read().decode("utf-8")
        needle = "Allowed origins are: ['o=3DDebian,n=3Dwheezy', "\
            "'o=3DDebian,n=3Dwheezy-updat=\n"\
            "es', 'o=3DDebian,n=3Dwheezy,l=3DDebian-Security', "\
            "'origin=3DDebian,archive=\n"\
            "=3Dstable,label=3DDebian-Security']"
        self.assertTrue(needle in log_data)


class SendmailAndMailxTestCase(SendmailTestCase):

    def setUp(self):
        self.common_setup()
        unattended_upgrade.MAIL_BINARY = make_mock_mailx(self.tmpdir)
        unattended_upgrade.SENDMAIL_BINARY = make_mock_sendmail(self.tmpdir)


if __name__ == "__main__":
    #logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import os
import subprocess
import unittest

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt

import unattended_upgrade

apt.apt_pkg.config.set("APT::Architecture", "amd64")


class MockOptions(object):
    debug = True
    verbose = False
    download_only = False
    dry_run = False
    apt_debug = False
    minimal_upgrade_steps = True


class TestRemoveUnused(unittest.TestCase):

    def setUp(self):
        self.rootdir = os.path.abspath("./root.unused-deps")
        # fake on_ac_power
        os.environ["PATH"] = (os.path.join(self.rootdir, "usr", "bin") + ":"
                              + os.environ["PATH"])
        dpkg_status = os.path.abspath(
            os.path.join(self.rootdir, "var", "lib", "dpkg", "status"))
        # fake dpkg status
        with open(dpkg_status, "w") as fp:
            fp.write("""Package: test-package
Status: install ok installed
Architecture: all
Version: 1.0.test.pkg
Depends: test-package-dependency

Package: test-package-dependency
Status: install ok installed
Architecture: all
Version: 1.0

Package: any-old-unused-modules
Status: install ok installed
Architecture: all
Version: 1.0

Package: linux-image-4.05.0-1021-kvm
Status: install ok installed
Architecture: all
Version: 1.21

Package: linux-image-4.05.0-1022-kvm
Status: install ok installed
Architecture: all
Version: 1.22

Package: linux-image-4.05.0-1023-kvm
Status: install ok installed
Architecture: all
Version: 1.23

Package: z-package
Status: install ok installed
Architecture: all
Version: 1.0

Package: old-unused-dependency
Status: install ok installed
Architecture: all
Version: 1.0
""")
        apt.apt_pkg.config.set("Dir::State::status", dpkg_status)
        apt.apt_pkg.config.clear("DPkg::Pre-Invoke")
        apt.apt_pkg.config.clear("DPkg::Post-Invoke")
        apt.apt_pkg.config.set("Debug::NoLocking", "true")
        # we don't really run dpkg
        apt.apt_pkg.config.set(
            "Dir::Bin::Dpkg", os.path.join(self.rootdir, "bin", "dpkg"))
        # pretend test-package-dependency is auto-installed
        extended_states = os.path.join(
            self.rootdir, "var", "lib", "apt", "extended_states")
        with open(extended_states, "w") as f:
            f.write("""
Package: old-unused-dependency
Architecture: all
Auto-Installed: 1

Package: test-package-dependency
Architecture: all
Auto-Installed: 1

Package: any-old-unused-modules
Architecture: all
Auto-Installed: 1

Package: linux-image-4.05.0-1021-kvm
Architecture: all
Auto-Installed: 1
""")
        # clean log
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")
        if not os.path.exists(os.path.dirname(self.log)):
            os.makedirs(os.path.dirname(self.log))
        with open(self.log, "w"):
            pass
        # clean cache
        subprocess.check_call(
            ["rm", "-f",
             os.path.join(self.rootdir, "var", "cache", "apt", "*.bin")])

    def test_remove_unused_dependencies(self):
        apt_conf = os.path.join(self.rootdir, "etc", "apt", "apt.conf")
        with open(apt_conf, "w") as fp:
            fp.write("""
Unattended-Upgrade::Keep-Debs-After-Install "true";
Unattended-Upgrade::Allowed-Origins {
    "Ubuntu:lucid-security";
};
Unattended-Upgrade::Remove-Unused-Dependencies "true";
Unattended-Upgrade::Skip-Updates-On-Metered-Connections "false";
""")
        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Ubuntu 10.04"
        unattended_upgrade.LOCK_FILE = "./u-u.lock"
        unattended_upgrade.main(
            options, rootdir="./root.unused-deps")
        with open(self.log) as f:
            # both the new and the old unused dependency are removed
            needle = "Packages that were successfully auto-removed: "\
                     "any-old-unused-modules linux-image-4.05.0-1021-kvm "\
                     "old-unused-dependency test-package-dependency\n"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))

    def test_remove_unused_dependencies_new_unused_only(self):
        apt.apt_pkg.config.set("APT::VersionedKernelPackages::", "linux-image")
        apt.apt_pkg.config.set("APT::VersionedKernelPackages::", ".*-modules")
        apt.apt_pkg.config.set("APT::VersionedKernelPackages::",
                               "linux-headers")
        apt_conf = os.path.join(self.rootdir, "etc", "apt", "apt.conf")
        with open(apt_conf, "w") as fp:
            fp.write("""
Unattended-Upgrade::Keep-Debs-After-Install "true";
Unattended-Upgrade::Allowed-Origins {
    "Ubuntu:lucid-security";
};
Unattended-Upgrade::Remove-New-Unused-Dependencies "true";
Unattended-Upgrade::Skip-Updates-On-Metered-Connections "false";
""")
        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Ubuntu 10.04"
        unattended_upgrade.LOCK_FILE = "./u-u.lock"
        unattended_upgrade.main(
            options, rootdir="./root.unused-deps")
        with open(self.log) as f:
            # ensure its only exactly one package that is removed
            needle_kernel_bad = "Removing unused kernel packages: "\
                                "any-old-unused-modules\n"
            needle_kernel_good = "Removing unused kernel packages: "\
                                 "linux-image-4.05.0-1021-kvm\n"
            needle = "Packages that were successfully auto-removed: "\
                     "test-package-dependency\n"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))
            self.assertTrue(needle_kernel_good in haystack,
                            "Can not find '%s' in '%s'" % (needle_kernel_good,
                                                           haystack))
            self.assertFalse(needle_kernel_bad in haystack,
                             "Found '%s' in '%s'" % (needle_kernel_bad,
                                                     haystack))

    def test_remove_valid(self):
        cache = unattended_upgrade.UnattendedUpgradesCache(
            rootdir=self.rootdir)
        auto_removable = unattended_upgrade.get_auto_removable(cache)
        print(auto_removable)
        cache["old-unused-dependency"].mark_delete()
        res = unattended_upgrade.is_autoremove_valid(
            cache, "test-package-dependency", auto_removable)
        self.assertTrue(res, "Simple autoremoval set is not valid")

        res = unattended_upgrade.is_autoremove_valid(
            cache, "test-package-dependency", set())
        self.assertFalse(res, "Autoremoving non-autoremovable package")

        cache["forbidden-dependency"].mark_install()
        auto_removable.add("forbidden-dependency")
        res = unattended_upgrade.is_autoremove_valid(
            cache, "test-package-dependency", auto_removable)
        self.assertFalse(
            res, "Package set to reinstall in cache is reinstalled")


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()



#!/usr/bin/python3

import os
import subprocess
import unittest


def hasMyPy():
    try:
        subprocess.check_call(["mypy", "--version"])
    except Exception:
        return False
    return True


class PackagePep484TestCase(unittest.TestCase):

    @unittest.skipIf(not hasMyPy(), "no mypy available")
    def test_pep484_clean(self):
        top_src_dir = os.path.join(os.path.dirname(__file__), "..")
        # FIXME: add pyi file for python-apt to get rid of the
        # --ignore-missing-imports
        self.assertEqual(
            subprocess.call(
                ["mypy", "--ignore-missing-imports",
                 "--no-strict-optional", top_src_dir]), 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import logging
import unittest

import unattended_upgrade
from unattended_upgrade import substitute, get_allowed_origins


class TestSubstitude(unittest.TestCase):

    def setUp(self):
        # monkey patch DISTRO_{CODENAME, ID}
        unattended_upgrade.DISTRO_CODENAME = "nacked"
        unattended_upgrade.DISTRO_ID = "MyDistroID"

    def testSubstitute(self):
        """ test if the substitute function works """
        self.assertTrue(substitute("${distro_codename}-updates"),
                        "nacked-updates")
        self.assertTrue(substitute("${distro_id}"), "MyDistroID")

    def test_get_allowed_origins_with_substitute(self):
        """ test if substitute for get_allowed_origins works """
        apt_pkg.config.clear("Unattended-Upgrade::Allowed-Origins")
        apt_pkg.config.set("Unattended-Upgrade::Allowed-Origins::",
                           "${distro_id} ${distro_codename}-security")
        li = get_allowed_origins()
        self.assertTrue(("o=MyDistroID,a=nacked-security") in li)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import io
import os
import sys
import tempfile
import unittest

from mock import (
    Mock,
    patch,
)

from unattended_upgrade import do_install

from typing import List


class MockCache(dict):
    blacklist = []  # type: List[str]
    whitelist = []  # type: List[str]

    def __iter__(self):
        raise StopIteration

    def get_changes(self):
        return []

    def mark_upgrade_adjusted(self, pkg, **kwargs):
        return


class TestRegression(unittest.TestCase):

    @unittest.skipIf(sys.version_info[0] != 3, "only works on py3")
    @patch("unattended_upgrade.upgrade_normal")
    def test_do_install_fail_unicode_write(self, mock_upgrade_normal):
        """ test if the substitute function works """
        def _raise(*args):
            raise Exception("meepä")
        tmp = tempfile.TemporaryFile()
        old_stderr = os.dup(2)
        # comment this line if you need to debug stuff
        os.dup2(tmp.fileno(), 2)

        mock_upgrade_normal.side_effect = _raise
        logfile_dpkg = io.StringIO()
        # mock pkg
        pkg = Mock()
        pkg.name = "meep"
        # mock options
        options = Mock()
        options.minimal_upgrade_steps = False
        apt_pkg.config.set("Unattended-Upgrade::MinimalSteps", "False")
        cache = MockCache()
        cache[pkg.name] = pkg
        do_install(cache=cache, pkgs_to_upgrade=[pkg.name],
                   options=options,
                   logfile_dpkg=logfile_dpkg)
        # if there is no exception here, we are good
        os.dup2(old_stderr, 2)
        tmp.seek(0)
        content = tmp.read().decode("utf-8").splitlines()
        tmp.close()
        self.assertEqual(content[-1], "Exception: meepä")


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import logging
import os
import shutil
import tempfile
import unittest

from unattended_upgrade import update_kept_pkgs_file


class MotdTestCase(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)
        self.addCleanup(logging.shutdown)

    def test_packages_kept(self):
        pkgs_kept_back = {"Debian wheezy-security": ["linux-image"],
                          "Debian wheezy": ["hello", "tworld"]}
        update_kept_pkgs_file(pkgs_kept_back, os.path.join(self.tmpdir,
                                                           "kept-back"))
        with open(os.path.join(self.tmpdir, "kept-back"), "rb") as fp:
            kept_txt = fp.read().decode("utf-8")
        self.assertEqual('hello linux-image tworld', kept_txt)
        update_kept_pkgs_file({}, os.path.join(self.tmpdir, "kept-back"))
        self.assertFalse(
            os.path.exists(os.path.join(self.tmpdir, "kept-back")))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt
import logging
import glob
import os
import re
import shutil
import subprocess
import sys
import time
import unittest

# debian
#SOURCES_LIST="""
#deb http://ftp.de.debian.org/debian squeeze main contrib non-free
#deb http://ftp.de.debian.org/debian squeeze-updates main contrib non-free
#deb http://ftp.de.debian.org/debian squeeze-proposed-updates main contrib \
#  non-free
#deb http://security.debian.org squeeze/updates main contrib non-free
#"""
#DISTRO="squeeze"
#ARCH="i386"
#TARBALL="%s-%s.tgz" % (DISTRO, ARCH)
#MIRROR="http://ftp.de.debian.org/debian"
#APT_CONF="""APT::Architecture "%s";""" % ARCH
#ORIGINS_PATTERN="origin=Debian,archive=stable,label=Debian-Security"


# ubuntu
SOURCES_LIST = """
deb http://archive.ubuntu.com/ubuntu/ lucid main restricted
deb-src http://archive.ubuntu.com/ubuntu/ lucid main restricted

deb http://archive.ubuntu.com/ubuntu/ lucid-updates main restricted
deb-src http://archive.ubuntu.com/ubuntu/ lucid-updates main restricted

deb http://security.ubuntu.com/ubuntu/ lucid-security main restricted
deb-src http://security.ubuntu.com/ubuntu/ lucid-security main restricted
"""
DISTRO = "lucid"
ARCH = "i386"
TARBALL = "%s-%s.tgz" % (DISTRO, ARCH)
MIRROR = "http://archive.ubuntu.com/ubuntu"
APT_CONF = """APT::Architecture "%s";""" % ARCH
ORIGINS_PATTERN = "origin=Ubuntu,archive=lucid-security"

apt.apt_pkg.config.set("APT::Architecture", ARCH)
sys.path.insert(0, "..")
import unattended_upgrade


class MockOptions(object):
    debug = False
    verbose = True
    apt_debug = False
    download_only = False
    dry_run = False
    minimal_upgrade_steps = True


class TestUnattendedUpgrade(unittest.TestCase):

    def _create_new_debootstrap_tarball(self, tarball, target):
        print("creating initial test tarball, this is needed only once")
        # force i386
        subprocess.call(
            ["debootstrap",
             "--arch=%s" % ARCH,
             # smaller version of the minimal system
             "--variant=minbase",
             "--include=python-apt,apt-utils,gpgv,ubuntu-keyring,"
             "ca-certificates",
             DISTRO,
             target,
             MIRROR])
        subprocess.call(["chroot", target, "apt-get", "clean"])
        subprocess.call(["tar", "czf", tarball, target])

    def _unpack_debootstrap_tarball(self, tarball, target):
        subprocess.call(["tar", "xzf", tarball])

    def test_normal_upgrade(self):
        print("Running normal unattended upgrade in chroot")
        options = MockOptions()
        options.minimal_upgrade_steps = True
        # run it
        target = self._run_upgrade_test_in_real_chroot(options)
        # ensure we upgraded the expected packages
        self.assertTrue(
            self._verify_install_log_in_real_chroot(target, "ca-certificates"))

    def test_non_minimal_steps_upgrade(self):
        print("Running non-minimal steps unattended upgrade in chroot")
        options = MockOptions()
        options.minimal_upgrade_steps = False
        # run it
        target = self._run_upgrade_test_in_real_chroot(options)
        # ensure we upgraded the expected packages
        self.assertTrue(
            self._verify_install_log_in_real_chroot(target, "ca-certificates"))

    def test_upgrade_on_shutdown_upgrade(self):
        print("Running unattended upgrade on shutdown (download and install) "
              "in chroot")
        # ensure that it actually installs in shutdown env mode
        options = MockOptions()
        os.environ["UNATTENDED_UPGRADES_FORCE_INSTALL_ON_SHUTDOWN"] = "1"
        apt.apt_pkg.config.set("Unattended-Upgrade::InstallOnShutdown", "1")
        target = self._run_upgrade_test_in_real_chroot(options)
        self.assertTrue(
            self._verify_install_log_in_real_chroot(target, "ca-certificates"))

    def test_whitelist_upgrade(self):
        print("Running unattended upgrade in chroot with whitelisted pkg")
        options = MockOptions()
        apt.apt_pkg.config.set(
            "Unattended-Upgrade::Package-Whitelist::", "^libc6$")
        # run it
        target = self._run_upgrade_test_in_real_chroot(options)
        # libc6 has a strict dependency on libc-bin so both packages
        # will have to get installed. ensure that the relaxed whitelist is ok
        self.assertTrue(
            self._verify_install_log_in_real_chroot(target, "libc6"))
        self.assertFalse(
            self._verify_install_log_in_real_chroot(target, "ca-certificates"))
        apt.apt_pkg.config.clear("Unattended-Upgrade::Package-Whitelist")

    def test_whitelist_upgrade_strict(self):
        print(
            "Running unattended upgrade in chroot with strict whitelisted pkg")
        options = MockOptions()
        apt.apt_pkg.config.set(
            "Unattended-Upgrade::Package-Whitelist::", "^apt$")
        apt.apt_pkg.config.set(
            "Unattended-Upgrade::Package-Whitelist::", "^libc6$")
        apt.apt_pkg.config.set(
            "Unattended-Upgrade::Package-Whitelist-Strict", "true")
        # run it
        target = self._run_upgrade_test_in_real_chroot(options)
        # we upgraded apt, but not libc6 because libc6 needs libc-bin and
        # its not whitelisted (in strict mode)
        self.assertTrue(
            self._verify_install_log_in_real_chroot(target, "apt"))
        self.assertFalse(
            self._verify_install_log_in_real_chroot(target, "libc6"))
        self.assertFalse(
            self._verify_install_log_in_real_chroot(target, "libc-bin"))
        apt.apt_pkg.config.clear("Unattended-Upgrade::Package-Whitelist")

    def _get_lockfile_location(self, target):
        return os.path.join(
            target, "var/log/unattended-upgrades/unattended-upgrades.log")

    def _setup_chroot(self, target):
        """ helper that setups a clean chroot """
        if os.path.exists(target):
            subprocess.call(["umount", os.path.join(target, "dev", "pts")])
            shutil.rmtree(target)
        if not os.path.exists(TARBALL):
            self._create_new_debootstrap_tarball(TARBALL, target)
        # create new
        self._unpack_debootstrap_tarball(TARBALL, target)
        open(os.path.join(target, "etc/apt/apt.conf"), "w").write(APT_CONF)
        open(os.path.join(target, "etc/apt/sources.list"), "w").write(
            SOURCES_LIST)

    def _run_upgrade_test_in_real_chroot(self, options, clean_chroot=True):
        """ helper that runs the unattended-upgrade in a chroot
            and does some basic verifications
        """
        if os.getuid() != 0:
            print("Skipping because uid != 0")
            return

        # clear to avoid pollution in the chroot
        apt.apt_pkg.config.clear("Acquire::http::ProxyAutoDetect")

        # create chroot
        target = "./test-chroot.%s" % DISTRO

        # setup chroot if needed
        if clean_chroot:
            self._setup_chroot(target)

        # ensure we have /dev/pts in the chroot
        ret = subprocess.call(["mount", "-t", "devpts", "devptsfs",
                               os.path.join(target, "dev", "pts")])
        if ret != 0:
            raise Exception("Failed to mount %s/proc" % target)
        self.addCleanup(
            lambda: subprocess.call(
                ["umount", os.path.join(target, "dev", "pts")]))

        # and run the upgrade test
        pid = os.fork()
        if pid == 0:
            # chroot
            os.chroot(target)
            os.chdir("/")
            if not os.path.exists("/var/log/unattended-upgrades/"):
                os.makedirs("/var/log/unattended-upgrades/")
            # make sure we are up-to-date
            subprocess.call(["apt-get", "update", "-q", "-q"])
            # run it
            apt.apt_pkg.config.clear("Unattended-Upgrade::Allowed-Origins")
            apt.apt_pkg.config.clear("Unattended-Upgrade::Origins-Pattern")
            apt.apt_pkg.config.set(
                "Unattended-Upgrade::Origins-Pattern::", ORIGINS_PATTERN)
            unattended_upgrade.DISTRO_CODENAME = DISTRO
            unattended_upgrade.main(options)
            os._exit(0)
        else:
            has_progress = False
            all_progress = ""
            last_progress = ""
            progress_log = os.path.join(
                target, "var/run/unattended-upgrades.progress")
            while True:
                time.sleep(0.01)
                if os.path.exists(progress_log):
                    progress = open(progress_log).read()
                    if progress and progress != last_progress:
                        has_progress = progress.startswith("Progress")
                        last_progress = progress
                    all_progress += progress
                # check exit status
                (apid, status) = os.waitpid(pid, os.WNOHANG)
                if pid == apid:
                    ret = os.WEXITSTATUS(status)
                    break
        #print("*******************", all_progress)
        self.assertEqual(ret, 0)
        # this number is a bit random, we just want to be sure we have
        # progress data
        self.assertTrue(has_progress, True)
        self.assertTrue(len(all_progress) > 5)
        return target

    def _verify_install_log_in_real_chroot(self, target, needle_pkg):
        # examine log
        log = self._get_lockfile_location(target)
        logfile = open(log).read()
        #print(logfile)
        if not re.search("Packages that will be upgraded:.*%s" % needle_pkg,
                         logfile):
            logging.warn(
                "Can not find expected %s upgrade in log" % needle_pkg)
            return False
        if "ERROR Installing the upgrades failed" in logfile:
            logging.warn("Got a ERROR in the logfile")
            return False
        dpkg_log = os.path.join(
            target, "var/log/unattended-upgrades/*-dpkg*.log")
        dpkg_logfile = open(glob.glob(dpkg_log)[0]).read()
        if not "Preparing to replace %s" % needle_pkg in dpkg_logfile:
            logging.warn("Did not find %s upgrade in %s" % (
                         dpkg_log, needle_pkg))
            return False
        #print(dpkg_logfile)
        return True


if __name__ == "__main__":
    #logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import unittest

from mock import Mock

from unattended_upgrade import calculate_upgradable_pkgs


class TestUnavailableCandidate(unittest.TestCase):

    def test_if_pkg_candidate_is_unavailable_then_pkg_is_not_considered(self):
        origin = Mock()
        origin.origin = 'allowed-origin'

        pkg = Mock()
        pkg.name = 'findutils'
        pkg.is_upgradable = False
        pkg.is_installed = True
        pkg.installed = Mock()
        pkg.installed.policy_priority = -1
        pkg.installed.origins = [origin]
        pkg.installed.version = '1:0.1'
        pkg.candidate = None
        pkg.versions = [pkg.installed]

        cache = Mock()
        cache.__iter__ = Mock(return_value=iter([pkg]))
        cache.allowed_origins = ["o=allowed-origin"]
        cache.blacklist = ['findutils']
        cache.whitelist = []
        options = Mock()

        pkgs_to_upgrade = \
            calculate_upgradable_pkgs(cache, options)

        self.assertListEqual([], pkgs_to_upgrade)


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import datetime
import logging
import os
import unittest
import subprocess

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")

from mock import (
    patch,
)

import unattended_upgrade


class RebootTestCase(unittest.TestCase):

    def setUp(self):
        # create reboot required file
        REBOOT_REQUIRED_FILE = "./reboot-required"
        with open(REBOOT_REQUIRED_FILE, "w"):
            pass
        self.addCleanup(lambda: os.remove(REBOOT_REQUIRED_FILE))
        unattended_upgrade.REBOOT_REQUIRED_FILE = REBOOT_REQUIRED_FILE
        # enable automatic-reboot
        apt_pkg.config.set("Unattended-Upgrade::Automatic-Reboot", "1")
        apt_pkg.config.set(
            "Unattended-Upgrade::Automatic-Reboot-WithUsers", "1")

    @patch("subprocess.check_output")
    def test_no_reboot_done_because_no_stamp(self, mock_call):
        unattended_upgrade.REBOOT_REQUIRED_FILE = "/no/such/file/or/directory"
        unattended_upgrade.reboot_if_requested_and_needed()
        self.assertEqual(mock_call.called, False)

    @patch("subprocess.check_output")
    def test_no_reboot_done_because_no_option(self, mock_call):
        apt_pkg.config.set("Unattended-Upgrade::Automatic-Reboot", "0")
        unattended_upgrade.reboot_if_requested_and_needed()
        self.assertEqual(mock_call.called, False)

    @patch("subprocess.check_output", return_value="some shutdown msg")
    @patch("unattended_upgrade.logged_in_users", return_value={})
    def test_reboot_now(self, mock_user, mock_call):
        unattended_upgrade.reboot_if_requested_and_needed()
        mock_call.assert_called_with(["/sbin/shutdown", "-r", "now"],
                                     stderr=subprocess.STDOUT)

    @patch("subprocess.check_output", return_value="some shutdown msg")
    @patch("unattended_upgrade.logged_in_users", return_value={})
    def test_reboot_time(self, mock_users, mock_call):
        apt_pkg.config.set(
            "Unattended-Upgrade::Automatic-Reboot-Time", "03:00")
        unattended_upgrade.reboot_if_requested_and_needed()
        mock_call.assert_called_with(["/sbin/shutdown", "-r", "03:00"],
                                     stderr=subprocess.STDOUT)

    @patch("subprocess.check_output", return_value="some shutdown msg")
    @patch("unattended_upgrade.logged_in_users", return_value={})
    def test_reboot_withoutusers(self, mock_users, mock_call):
        """Ensure that a reboot happens when no users are logged in"""
        apt_pkg.config.set(
            "Unattended-Upgrade::Automatic-Reboot-WithUsers", "0")
        apt_pkg.config.set(
            "Unattended-Upgrade::Automatic-Reboot-Time", "04:00")
        # some pgm that allways output nothing
        unattended_upgrade.reboot_if_requested_and_needed()
        mock_call.assert_called_with(["/sbin/shutdown", "-r", "04:00"],
                                     stderr=subprocess.STDOUT)

    @patch("subprocess.check_output")
    @patch("unattended_upgrade.logged_in_users", return_value={'user'})
    def test_reboot_withusers(self, mock_users, mock_call):
        """Ensure that a reboot does not happen if a user is logged in"""
        apt_pkg.config.set(
            "Unattended-Upgrade::Automatic-Reboot-WithUsers", "0")
        # some pgm that allways output a word
        unattended_upgrade.reboot_if_requested_and_needed()
        self.assertEqual(
            mock_call.called, False,
            "Called '%s' when nothing should have "
            "happen" % mock_call.call_args_list)

    @patch("subprocess.call")
    def test_logged_in_users(self, mock_call):
        # some pgm that allways output a word
        unattended_upgrade.USERS = ["/bin/date", "+%Y %Y %Y"]
        users = unattended_upgrade.logged_in_users()
        today = datetime.date.today()
        self.assertEqual(users, set([today.strftime("%Y")]))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
#
# create a lock file so that unattended-upgrades-shutdown pauses
# on shutdown -- useful for testing

import apt_pkg
import os
import time


pid = os.fork()
if pid == 0:
    os.setsid()
    lock = apt_pkg.get_lock("/var/run/unattended-upgrades.lock")
    time.sleep(500)



#!/usr/bin/python3

import datetime
import logging
import os
import unittest

import apt

import unattended_upgrade

apt.apt_pkg.config.set("APT::Architecture", "amd64")


class MockOptions(object):
    debug = True
    verbose = False
    dry_run = True
    apt_debug = False
    minimal_upgrade_steps = False
    download_only = False


class MockDistroAuto(object):

    start_updates = datetime.date.today()
    release = (datetime.date.today()
               + unattended_upgrade.DEVEL_UNTIL_RELEASE
               + datetime.timedelta(days=1))
    series = "artful"

    def devel(self, result):
        assert result == "object"
        return self


class MockDistroNoAuto(object):

    release = (datetime.date.today()
               + unattended_upgrade.DEVEL_UNTIL_RELEASE)
    series = "artful"

    def devel(self, result):
        assert result == "object"
        return self


class MockDistroNoRelease(object):

    release = None
    series = "artful"

    def devel(self, result):
        assert result == "object"
        return self


class MockDistroInfoModule(object):

    def __init__(self, ubuntu):
        self.UbuntuDistroInfo = ubuntu


class TestUntrusted(unittest.TestCase):

    def setUp(self):
        apt.apt_pkg.config.set("Unattended-Upgrade::"
                               "Skip-Updates-On-Metered-Connections",
                               "false")
        apt.apt_pkg.config.set("Unattended-Upgrade::OnlyOnAcPower",
                               "false")
        unattended_upgrade.LOCK_FILE = "./u-u.lock"
        self.rootdir = os.path.abspath("./root.untrusted")
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")

        self.apt_conf = os.path.join(self.rootdir, "etc", "apt",
                                     "apt.conf")

        os.rename(self.apt_conf, self.apt_conf + ".bak")
        for hdlr in logging.root.handlers:
            hdlr.close()
        logging.root.handlers = []

    def tearDown(self):
        os.remove(self.log)
        os.rename(self.apt_conf + ".bak", self.apt_conf)

    def write_config(self, devrelease):
        with open(self.apt_conf, "w") as fp:
            fp.write("""Unattended-Upgrade::DevRelease "%s";
Unattended-Upgrade::Skip-Updates-On-Metered-Connections "false";
Unattended-Upgrade::OnlyOnAcPower "false";
""" % devrelease)

    def test_do_not_run_on_devrelease(self):
        self.write_config("false")

        # run it
        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Artful Aardvark (development branch)"
        unattended_upgrade.main(options, rootdir=self.rootdir)
        # read the log to see what happend
        with open(self.log) as f:
            needle = "Not running on the development release"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))

    def test_auto_devrelease(self):
        """We are not ready to auto upgrade just yet"""
        self.write_config("auto")

        # run it
        unattended_upgrade.distro_info = MockDistroInfoModule(MockDistroAuto)

        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Artful Aardvark (development branch)"
        unattended_upgrade.DISTRO_CODENAME = "artful"
        unattended_upgrade.DISTRO_ID = "ubuntu"
        unattended_upgrade.main(options, rootdir=self.rootdir)
        # read the log to see what happend
        with open(self.log) as f:
            needle = ("Not running on this development release before %s"
                      % MockDistroAuto.start_updates)
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))

    def test_noauto_devrelease(self):
        """We are in the valid time period"""
        self.write_config("auto")
        # run it
        unattended_upgrade.distro_info = MockDistroInfoModule(MockDistroNoAuto)

        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Artful Aardvark (development branch)"
        unattended_upgrade.DISTRO_CODENAME = "artful"
        unattended_upgrade.DISTRO_ID = "ubuntu"
        unattended_upgrade.main(options, rootdir=self.rootdir)
        # read the log to see what happend
        with open(self.log) as f:
            # Check that we could have run
            needle = "Running on the development release"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))

    def test_norelease_devrelease(self):
        """The devel series has no release update, so do updates"""
        self.write_config("auto")
        # run it
        unattended_upgrade.distro_info = MockDistroInfoModule(MockDistroNoAuto)

        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Artful Aardvark (development branch)"
        unattended_upgrade.DISTRO_CODENAME = "artful"
        unattended_upgrade.DISTRO_ID = "ubuntu"
        unattended_upgrade.main(options, rootdir=self.rootdir)
        # read the log to see what happend
        with open(self.log) as f:
            # Check that we could have run
            needle = "Running on the development release"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()



#!/usr/bin/python3

import os
import logging
import shutil
import unittest

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")

from unattended_upgrade import (
    conffile_prompt,
    dpkg_conffile_prompt,
)


class ConffilePromptTestCase(unittest.TestCase):

    def setUp(self):
        apt_pkg.config.set("Dir::State::status",
                           "./root.conffile/var/lib/dpkg/status")
        with open("./root.conffile/etc/configuration-file", "w") as fp:
            fp.write("""This is a configuration file,
dfasddfasdff
No really.
""")

    def tearDown(self):
        try:
            os.remove("./root.conffile/etc/configuration-file")
        except Exception:
            pass
        try:
            shutil.rmtree("./root.conffile/etc/configuration-file")
        except Exception:
            pass

    def test_will_prompt(self):
        # conf-test 0.9 is installed, 1.1 gets installed
        # they both have different config files
        test_pkg = "./packages/conf-test-package_1.1.deb"
        self.assertTrue(conffile_prompt(test_pkg, prefix="./root.conffile"),
                        "conffile prompt detection incorrect")

    def test_will_prompt_on_moves(self):
        # changed /etc/foo becomes different /etc/foo/foo
        test_pkg = "./packages/test-package_1.2_all.deb"
        self.assertTrue(conffile_prompt(test_pkg, prefix="./root.conffile"),
                        "conffile prompt detection incorrect")
        # changed /etc/foo/foo becomes different /etc/foo
        test_pkg = "./packages/test-package-2_1.2_all.deb"
        self.assertTrue(conffile_prompt(test_pkg, prefix="./root.conffile"),
                        "conffile prompt detection incorrect")

    def test_prompt_on_deleted_modified_conffile(self):
        # conf-test 0.9 is installed, 1.1 gets installed
        # they both have different config files, this triggers a prompt
        os.remove("./root.conffile/etc/configuration-file")
        test_pkg = "./packages/conf-test-package_1.1.deb"
        self.assertTrue(conffile_prompt(test_pkg, prefix="./root.conffile"),
                        "conffile prompt detection incorrect")

    def test_will_not_prompt(self):
        # conf-test 0.9 is installed, 1.0 gets installed
        # they both have the same config files
        test_pkg = "./packages/conf-test-package_1.0.deb"
        self.assertFalse(conffile_prompt(test_pkg, prefix="./root.conffile"),
                         "conffile prompt detection incorrect")

    def test_will_not_prompt_on_moves(self):
        # changed /etc/foo becomes /etc/foo/foo, same as shipped before
        test_pkg = "./packages/test-package_1.3_all.deb"
        self.assertFalse(conffile_prompt(test_pkg, prefix="./root.conffile"),
                         "conffile prompt detection incorrect")
        # changed /etc/foo/foo becomes /etc/foo, same as shipped before
        test_pkg = "./packages/test-package-2_1.3_all.deb"
        self.assertFalse(conffile_prompt(test_pkg, prefix="./root.conffile"),
                         "conffile prompt detection incorrect")

    def test_with_many_entries(self):
        # ensure we don't crash when encountering a conffile with overly
        # many entries
        test_pkg = "./packages/conf-test-package-257-conffiles_1.deb"
        self.assertFalse(conffile_prompt(test_pkg, prefix="./root.conffile"),
                         "conffile prompt detection incorrect")

    def test_will_not_prompt_because_of_conffile_removal(self):
        # no conffiles anymore in the pkg
        test_pkg = "./packages/conf-test-package-no-conffiles-anymore_2.deb"
        self.assertFalse(conffile_prompt(test_pkg, prefix="./root.conffile"),
                         "conffile prompt detection incorrect")

    def test_will_prompt_multiple(self):
        # multiple conffiles
        test_pkg = "./packages/multiple-conffiles_2_all.deb"
        self.assertTrue(conffile_prompt(test_pkg, prefix="./root.conffile"),
                        "conffile prompt detection incorrect")

    def test_will_prompt_for_new_conffile(self):
        # debian bug #673237, a package that was not a conffile now
        # becomes a conffile
        test_pkg = "./packages/conf-test-package-new-conffile_1.deb"
        self.assertTrue(conffile_prompt(test_pkg, prefix="./root.conffile"),
                        "conffile prompt detection incorrect")

    def test_xz_compression(self):
        test_pkg = "./packages/conf-test-xz_1.0_all.deb"
        self.assertFalse(
            conffile_prompt(test_pkg, prefix="./root.conffile"),
            "conffile prompt detection incorrect")


class DpkgConffileTestCase(unittest.TestCase):
    """
    This tests that the detection if dpkg will prompt at all works,
    i.e. if the user has decided to use a --force-conf{old,new} option
    """

    def setUp(self):
        apt_pkg.config.clear("DPkg::Options")

    def test_no_dpkg_prompt_option(self):
        self.assertTrue(dpkg_conffile_prompt())

    def test_regression_lp1061498(self):
        apt_pkg.config.set("DPkg::Options::", "muup")
        self.assertTrue(dpkg_conffile_prompt())

    def test_dpkg_will_never_prompt(self):
        apt_pkg.config.set("DPkg::Options::", "--force-confold")
        self.assertFalse(dpkg_conffile_prompt())


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/env python3

import glob
from setuptools import setup
from DistUtilsExtra.command import (
    build_extra,
    build_i18n,
)

if __name__ == "__main__":
    setup(
        name='unattended-upgrades',
        version='0.1',
        scripts=['unattended-upgrade'],
        data_files=[
            ('../etc/logrotate.d/',
                ["data/logrotate.d/unattended-upgrades"]),
            ('../etc/update-motd.d/',
                ["data/92-unattended-upgrades"]),
            ('../usr/share/unattended-upgrades/',
                ["data/20auto-upgrades",
                 "data/20auto-upgrades-disabled",
                 "data/50unattended-upgrades",
                 "data/50unattended-upgrades.md5sum",
                 "unattended-upgrade-shutdown",
                 "data/update-motd-unattended-upgrades"]),
            ('../usr/share/man/man8/',
                ["man/unattended-upgrade.8"]),
            ('../etc/pm/sleep.d/',
                ["pm/sleep.d/10_unattended-upgrades-hibernate"]),
            ('../etc/kernel/postinst.d/',
                glob.glob("kernel/postinst.d/*")),
            ('../lib/systemd/system-sleep/',
                ["debian/systemd-sleep/unattended-upgrades"]),
            ('../usr/share/apport/package-hooks/',
                ["debian/source_unattended-upgrades.py"])
        ],
        cmdclass={"build": build_extra.build_extra,
                  "build_i18n": build_i18n.build_i18n},
        test_suite="test",
    )



#!/usr/bin/python3

import os
import subprocess
import unittest

# E126: I don't even know what its supposed to tell me :(
IGNORE = "E126,E265,E402,W503"


class PackagePep8TestCase(unittest.TestCase):

    def test_pep8_clean(self):
        top_src_dir = os.path.join(os.path.dirname(__file__), "..")
        targets = [
            top_src_dir,
            os.path.join(top_src_dir, "unattended-upgrade"),
            os.path.join(top_src_dir, "unattended-upgrade-shutdown"),
        ]
        try:
            self.assertEqual(subprocess.call(["pycodestyle",
                                              "--repeat",
                                              "--ignore=%s"
                                              % IGNORE]
                                             + targets),
                             0)
        except FileNotFoundError:
            self.assertEqual(subprocess.call(
                ["pep8", "--repeat", "--ignore=%s" % IGNORE] + targets), 0)


if __name__ == "__main__":
    unittest.main()



"""
unattended-upgrades apport hook

Collects the following log file:
  - /var/log/unattended-upgrades/unattended-upgrades.log
Check to see if either of these conffiles has been modified:
  - /etc/apt/apt.conf.d/50unattended-upgrades
  - /etc/apt/apt.conf.d/10periodic
"""

from apport.hookutils import (
    attach_conffiles,
    attach_file_if_exists)


def add_info(report, ui):
    # always attach these files
    attach_conffiles(report, 'unattended-upgrades', ui=ui)
    attach_conffiles(report, 'update-notifier-common', ui=ui)
    attach_file_if_exists(
        report, '/var/log/unattended-upgrades/unattended-upgrades.log')
    attach_file_if_exists(
        report, '/var/log/apt/history.log')



#!/usr/bin/python3

from datetime import date
import unittest
from mock import patch

import apt

import unattended_upgrade


class TestUpdateDays(unittest.TestCase):

    def setUp(self):
        apt.apt_pkg.config.clear("Unattended-Upgrade::Update-Days")

    def test_update_days_no_patch_days_always_runs_uu(self):
        apt.apt_pkg.config.clear("Unattended-Upgrade::Update-Days")
        self.assertTrue(unattended_upgrade.is_update_day())

    @patch("unattended_upgrade.date")
    def test_update_days_by_name(self, mock_date):
        mock_date.side_effect = lambda *args, **kw: date(*args, **kw)
        apt.apt_pkg.config.set("Unattended-Upgrade::Update-Days::", "Wed")
        apt.apt_pkg.config.set("Unattended-Upgrade::Update-Days::", "Sun")
        # that was a Wed
        mock_date.today.return_value = date(2007, 8, 22)
        self.assertTrue(unattended_upgrade.is_update_day())
        # that was a Th
        mock_date.today.return_value = date(2007, 8, 2)
        self.assertFalse(unattended_upgrade.is_update_day())
        # that was a Sun
        mock_date.today.return_value = date(2007, 7, 29)
        self.assertTrue(unattended_upgrade.is_update_day())

    @patch("unattended_upgrade.date")
    def test_update_days_by_number(self, mock_date):
        mock_date.side_effect = lambda *args, **kw: date(*args, **kw)
        # 0: Sun, 1: Mon, ...
        apt.apt_pkg.config.set("Unattended-Upgrade::Update-Days::", "3")
        apt.apt_pkg.config.set("Unattended-Upgrade::Update-Days::", "0")
        # that was a Wed
        mock_date.today.return_value = date(2007, 8, 22)
        self.assertTrue(unattended_upgrade.is_update_day())
        # that was a Thu
        mock_date.today.return_value = date(2007, 8, 2)
        self.assertFalse(unattended_upgrade.is_update_day())
        # that was a Sun
        mock_date.today.return_value = date(2007, 7, 29)
        self.assertTrue(unattended_upgrade.is_update_day())


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3

import os
import sys
import subprocess


def test_systemd_service():
    '''
    Verify that the unattended-upgrades.service unit is started
    correctly. The unit must be started in order for the ExecStop=
    to work correctly
    '''
    Service = 'unattended-upgrades.service'
    try:
        subprocess.check_output(['systemctl', '--quiet', 'is-active', Service])
    except subprocess.CalledProcessError:
        out = subprocess.getoutput(
            'systemctl status unattended-upgrades.service')
        print('test_systemd_service() FAILED\n%s' % out)
        return False
    return True


def enable_install_on_shutdown():
    '''
    Enable InstallOnShutdown to verify that the command runs correctly
    upon reboot
    '''
    apt_conf_file = '/etc/apt/apt.conf.d/50unattended-upgrades'
    param = 'Unattended-Upgrade::InstallOnShutdown'
    sed_cmd = 's/\\/\\/%s.*/%s "true";/' % (param, param)

    try:
        subprocess.check_output(['/bin/sed', '-i', sed_cmd, apt_conf_file])
    except subprocess.CalledProcessError:
        print("Unable to edit %s" % apt_conf_file)
        return False
    return True


def check_log_files():
    '''
    Verify that the logfiles are correctly produced by the InstallOnShutdown
    run upon reboot. This will confirm that it did run correctly when we
    rebooted.
    '''
    logdir = '/var/log/unattended-upgrades/'
    logfiles = ['unattended-upgrades.log', 'unattended-upgrades-shutdown.log']

    for file in logfiles:
        if not os.path.exists(logdir + file):
            print("File missing : %s" % (logdir + file))
            return False
    return True


if __name__ == '__main__':
    autopkgtest_reboot_mark = os.getenv('AUTOPKGTEST_REBOOT_MARK')

    if autopkgtest_reboot_mark is None:
        if not test_systemd_service():
            sys.exit(1)

        if enable_install_on_shutdown():
            print('Rebooting to test InstallOnShutdown...')
            subprocess.check_call(['/tmp/autopkgtest-reboot-prepare',
                                   'InstallOnShutdown'])
            subprocess.check_call(['dbus-send', '--system', '--print-reply',
                                   '--dest=org.freedesktop.login1',
                                   '/org/freedesktop/login1',
                                   'org.freedesktop.login1.Manager.Reboot',
                                   'boolean:false'])
        else:
            sys.exit(1)

    if autopkgtest_reboot_mark == 'InstallOnShutdown':
        if not check_log_files():
            print("InstallOnShutdown did not run")
            sys.exit(1)
    else:
        print('Invalid autopkgtest_reboot_mark value')
        sys.exit(1)

    sys.exit(0)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

import sys
import unittest

from mock import (
    Mock,
)

from unattended_upgrade import calculate_upgradable_pkgs


class TestBlacklistedWrongOrigin(unittest.TestCase):

    @unittest.skipIf(sys.version_info[0] != 3, "only works on py3")
    def test_if_origin_does_not_match_then_blacklist_is_not_checked(self):
        origin = Mock()
        origin.origin = "some-other-origin"

        pkg = Mock()
        pkg.name = "postgresql"
        pkg.is_upgradable = True
        pkg.candidate = Mock()
        pkg.candidate.policy_priority = 500
        pkg.candidate.origins = [origin]
        pkg.versions = [pkg.candidate]

        cache = Mock()
        cache.__iter__ = Mock(return_value=iter([pkg]))
        options = Mock()
        cache.allowed_origins = ["o=allowed-origin"]
        cache.blacklist = ["postgresql"]
        cache.whitelist = []

        pkgs_to_upgrade = calculate_upgradable_pkgs(cache, options)

        self.assertListEqual([], pkgs_to_upgrade)


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import os
import unittest

import apt_pkg
apt_pkg.config.set("Dir", "./aptroot")
import apt

import unattended_upgrade

apt.apt_pkg.config.set("APT::Architecture", "amd64")


class MockOptions(object):
    debug = False
    verbose = False
    download_only = False
    dry_run = True
    apt_debug = False
    minimal_upgrade_steps = True


class OnBattery(unittest.TestCase):

    def setUp(self):
        self.rootdir = os.path.abspath("./root.on-battery")
        # fake on_ac_power
        os.environ["PATH"] = (os.path.join(self.rootdir, "usr", "bin") + ":"
                              + os.environ["PATH"])
        dpkg_status = os.path.abspath(
            os.path.join(self.rootdir, "var", "lib", "dpkg", "status"))
        apt.apt_pkg.config.set("Dir::State::status", dpkg_status)
        apt.apt_pkg.config.clear("DPkg::Pre-Invoke")
        apt.apt_pkg.config.clear("DPkg::Post-Invoke")
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")

    def tearDown(self):
        os.remove(self.log)

    def test_on_battery(self):
        # ensure there is no conffile_prompt check

        # run it
        options = MockOptions()
        unattended_upgrade.DISTRO_DESC = "Ubuntu 10.04"
        unattended_upgrade.LOCK_FILE = "./u-u.lock"
        ret = unattended_upgrade.main(options, rootdir=self.rootdir)
        self.assertTrue(ret == 1)
        # read the log to see what happend
        with open(self.log) as f:
            needle = "System is on battery power, stopping"
            haystack = f.read()
            self.assertTrue(needle in haystack,
                            "Can not find '%s' in '%s'" % (needle, haystack))


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()
